﻿------------------------------------------------------------------------------------------
--Setup the Class Power-------------------------------------------------------------------
------------------------------------------------------------------------------------------
local SN, L, F, T, M, _ = unpack(select(2,...))
local LSM = LibStub("LibSharedMedia-3.0")
local MODNAME = "classResources"
local CP = SN:NewModule(MODNAME, "AceEvent-3.0")

local UnitHasVehicleUI = UnitHasVehicleUI
local unpack = unpack

SN["classMaxResourceBar"] = {
	["DEATHKNIGHT"] = 6,
	["PALADIN"] = 5,
	["WARLOCK"] = 4,
	["PRIEST"] = 3,
	["MONK"] = 5,
	["MAGE"] = 6,
}
local classResources

local function getOptions()
	classResources = {
		type = "group",
		name = L["Class Power"],
		args = {
			enable = {
				order = 0,
				type = "toggle",
				name = L["Enable Class Power"],
				get = function(info)
					return SN.data.profile.classResources[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.classResources[info[#info]] = value
					if value then
						SN:EnableModule(MODNAME)
					else
						SN:DisableModule(MODNAME)
					end
				end,
						
			},
			orientation = {
				order = 1,
				type = "select",
				name = L["Rune Fill Direction"],
				values = {
					["HORIZONTAL"] = "HORIZONTAL",
					["VERTICAL"] = "VERTICAL",
				},
				hidden = function() return SN.playerClass ~= "DEATHKNIGHT" or
						not SN.data.profile.classResources.enable end,
				get = function(info)
					return SN.data.profile.classResources[info[#info]]
				end,
				set = function(info, value)
					for i = 1, 6 do
						F.classpower.runes[i]:SetOrientation(value)
					end
					SN.data.profile.classResources[info[#info]] = value							
				end,
			},
			h = {
				order = 2,
				type = "header",
				name = "Dimensions",
				hidden = function() return not SN.data.profile.classResources.enable end,
			},
			width = {
				type = "range",
				name = L["Width of Class Power"],
				hidden = function() return not SN.data.profile.classResources.enable end,
				order = 4,
				min = 50, max = 500, step = 1,
				get = function(info)
					return SN.data.profile.classResources[info[#info]] 
				end,
				set = function(info, value)
					SN.data.profile.classResources[info[#info]] = value
					CP:ClassResize()
				end,
			},
			height = {
				type = "range",
				name = L["Height of Class Power"],
				hidden = function() return not SN.data.profile.classResources.enable end,
				order = 5,
				min = 10, max = 100, step = 1,
				get = function(info)
					return SN.data.profile.classResources[info[#info]] 
				end,
				set = function(info, value)
					SN.data.profile.classResources[info[#info]] = value
					CP:ClassResize()
				end,
			},
			texture = {
				type = "select",
				dialogControl = 'LSM30_Statusbar',
				name = L["Texture"],
				hidden = function() return not SN.data.profile.classResources.enable end,
				order = 14,
				values = AceGUIWidgetLSMlists.statusbar,
				get = function(info) return SN.data.profile.classResources[info[#info]] end,
				set = function(info, value)
					SN.data.profile.classResources[info[#info]] = value
					CP:ClassReskin(value)
				end,
			},
		},
	}
	return classResources
end

function CP:OnInitialize()
	if SN.playerClass == "HUNTER" or SN.playerClass == "SHAMAN" or SN.playerClass == "WARRIOR" or SN.playerClass == "MAGE" then
		CP:SetEnabledState(false)
		SN.data.profile.classResources.enable = false
	else
		SN:RegisterModuleOptions(MODNAME, getOptions, L["Class Power"])
	
		F.classpower = CreateFrame("Frame", nil, UIParent)
		F.classpower:SetPoint(unpack(SN.data.profile.frames.classResources))
		F.classpower:SetSize(SN.data.profile.classResources.width, SN.data.profile.classResources.height)
		F.classpower.name = "Class Power"
		F.classpower.db = "classResources"

		F.classpower.value = F.classpower:CreateFontString(nil, "OVERLAY")
		F.classpower.value:SetPoint("CENTER", F.classpower, "CENTER", 4, 0)
		F.classpower.value:SetFontObject(GameFontNormal)
		F.classpower.value:SetJustifyH("CENTER")
		F.classpower.value:SetShadowOffset(1, -1)
		F.classpower:SetScript("OnEvent", function(self, event, ...)
			if event == "PLAYER_ENTERING_WORLD" then
				if SN.data.profile.classResources.enable then
					if SN.playerClass == "DEATHKNIGHT" then
						CP:RuneEnable()
					elseif SN.playerClass == "PALADIN" then
						CP:HolyEnable()
					elseif SN.playerClass == "DRUID" or SN.playerClass == "ROGUE" then
						if SN.playerClass == "DRUID" then
							CP:EclipseEnable()
						end
						CP:ComboEnable()
					elseif SN.playerClass == "MONK" then
						CP:ChiEnable()
					elseif SN.playerClass == "PRIEST" then
						CP:SOrbsEnable()
					elseif SN.playerClass == "WARLOCK" then
						CP:ShardsEnable()
					end
				end
				
				self:UnregisterEvent("PLAYER_ENTERING_WORLD")
			end
			if UnitHasVehicleUI("player") then
				self:Hide()
			else
				self:Show()
			end
		end)
		if SN.playerClass == "DEATHKNIGHT" then
			CP:SetupRunes()
		elseif SN.playerClass == "PALADIN" then
			CP:SetupHoly()
		elseif SN.playerClass == "DRUID" or SN.playerClass == "ROGUE" then
			if SN.playerClass == "DRUID" then
				CP:SetupEclipse()
			end
			CP:SetupCombo()
		elseif SN.playerClass == "MONK" then
			CP:SetupChi()
		elseif SN.playerClass == "PRIEST" then
			CP:SetupSOrbs()
		elseif SN.playerClass == "WARLOCK" then
			CP:SetupShards()
		end
		F.classpower:RegisterEvent("PLAYER_ENTERING_WORLD")
		F.classpower:Hide()
	end
	
end

function CP:OnEnable()
	if SN.data.profile.classResources.enable then
		F.classpower:Show()
		F.classpower:RegisterEvent("UNIT_DISPLAYPOWER")
		F.classpower:RegisterUnitEvent("UNIT_ENTERED_VEHICLE", "player")
		F.classpower:RegisterUnitEvent("UNIT_EXITED_VEHICLE", "player")
		
		if SN.playerClass == "DEATHKNIGHT" then
			CP:RuneEnable()
		elseif SN.playerClass == "PALADIN" then
			CP:HolyEnable()
		elseif SN.playerClass == "DRUID" or SN.playerClass == "ROGUE" then
			if SN.playerClass == "DRUID" then
				CP:EclipseEnable()
			end
			CP:ComboEnable()
		elseif SN.playerClass == "MONK" then
			CP:ChiEnable()
		elseif SN.playerClass == "PRIEST" then
			CP:SOrbsEnable()
		elseif SN.playerClass == "WARLOCK" then
			CP:ShardsEnable()
		end
		if IsAddOnLoaded("ElvUI") then
			SN:ElvOverride()
		end
	end
end

function CP:OnDisable()
	F.classpower:UnregisterAllEvents()
	if SN.playerClass == "DEATHKNIGHT" then
		CP:RuneDisable()
	elseif SN.playerClass == "PALADIN" then
		CP:HolyDisable()
	elseif SN.playerClass == "DRUID" or SN.playerClass == "ROGUE" then
		if SN.playerClass == "DRUID" then
			CP:EclipseDisable()
		end
		CP:ComboDisable()
	elseif SN.playerClass == "MONK" then
		CP:ChiDisable()
	elseif SN.playerClass == "PRIEST" then
		CP:SOrbsDisable()
	elseif SN.playerClass == "WARLOCK" then
		CP:ShardsDisable()
	end
	
	F.classpower:Hide()
	if IsAddOnLoaded("ElvUI") then
		SN:ElvOverride()
	end
end

function CP:Lock()
	SN:LockFrame(F.classpower)
	if SN.data.profile.classResources.enable then
		if SN.playerClass == "DEATHKNIGHT" then
			F.classpower.runes:Show()
		elseif SN.playerClass == "PALADIN" then
			F.classpower.holypower:Show()
		elseif SN.playerClass == "ROGUE" then
			F.classpower.points:Show()
		elseif SN.playerClass == "DRUID" then
			local form = GetShapeshiftFormID()
			if  form == MOONKIN_FORM then
				F.classpower.eclipse:Show()
			elseif form == CAT_FORM then
				F.classpower.points:Show()
			end
		elseif SN.playerClass == "MONK" then
			F.classpower.chi:Show()
		elseif SN.playerClass == "PRIEST" then
			F.classpower.shadoworbs:Show()
		elseif SN.playerClass == "WARLOCK" then
			F.classpower.shards:Show()
		end
	end
end

function CP:Unlock()
	SN:UnlockFrame(F.classpower)
	if SN.data.profile.classResources.enable then
		if SN.playerClass == "DEATHKNIGHT" then
			F.classpower.runes:Hide()
		elseif SN.playerClass == "PALADIN" then
			F.classpower.holypower:Hide()
		elseif SN.playerClass == "DRUID" or SN.playerClass == "ROGUE" then
			if SN.playerClass == "DRUID" then
				F.classpower.eclipse:Hide()
			end
			F.classpower.points:Hide()
		elseif SN.playerClass == "MONK" then
			F.classpower.chi:Hide()
		elseif SN.playerClass == "PRIEST" then
			F.classpower.shadoworbs:Hide()
		elseif SN.playerClass == "WARLOCK" then
			F.classpower.shards:Hide()
		end
	end
end

function CP:Reset()
	local static = SN.DB.profile.frames
	local temp = SN.data.profile.frames
	F.classpower:ClearAllPoints()

	F.classpower:SetPoint(unpack(static.classResources))
	wipe(temp.classResources)
end

function CP:ApplySettings()
	F.classpower:SetPoint(unpack(SN.data.profile.frames.classResources))
	CP:ClassResize()
	if F.classpower.runes ~= nil then
		for i = 1, F.classpower.runes.maxRunes do
			F.classpower.runes[i]:SetOrientation(SN.data.profile.classResources.orientation)
		end
	end
	CP:ClassReskin(SN.data.profile.classResources.texture)
	SN:ElvOverride()
end

function CP:ClassResize()
	local MAX_HOLY_POWER = 5
	local MAX_COMBO_POINTS = MAX_COMBO_POINTS
	local maxRune = SN['classMaxResourceBar']["DEATHKNIGHT"]
	local maxChi = SN['classMaxResourceBar']["MONK"]
	
	F.classpower:SetSize(SN.data.profile.classResources.width, SN.data.profile.classResources.height)

	if SN.playerClass == "DEATHKNIGHT" then
		F.classpower.runes:SetSize(F.classpower:GetWidth(), F.classpower:GetHeight())
		for i = 1, maxRune do
			F.classpower.runes[i]:SetSize((F.classpower.runes:GetWidth() - 21) / 6, F.classpower.runes:GetHeight() - 6)
		end
	elseif SN.playerClass == "PALADIN" then
		F.classpower.holypower:SetSize(F.classpower:GetWidth(), F.classpower:GetHeight())
		for i = 1, MAX_HOLY_POWER do
			F.classpower.holypower[i]:SetSize((F.classpower.holypower:GetWidth() - 18) / MAX_HOLY_POWER, F.classpower.holypower:GetHeight() - 6)
		end
	elseif SN.playerClass == "DRUID" or SN.playerClass == "ROGUE" then
		if SN.playerClass == "DRUID" and GetSpecialization() == 1 then
			F.classpower.eclipse:SetSize(F.classpower:GetWidth(), F.classpower:GetHeight())
			F.classpower.eclipse[1]:SetSize(F.classpower.eclipse:GetWidth() - 1, F.classpower.eclipse:GetHeight() - 1)
			F.classpower.eclipse[2]:SetSize(F.classpower.eclipse:GetWidth() - 1, F.classpower.eclipse:GetHeight() - 1)
		end
		F.classpower.points:SetSize(F.classpower:GetWidth(), F.classpower:GetHeight())
		for i = 1, MAX_COMBO_POINTS do
			F.classpower.points[i]:SetSize((F.classpower.points:GetWidth() - 18) / MAX_COMBO_POINTS, F.classpower.points:GetHeight() - 6)
		end
	elseif SN.playerClass == "MONK" then
		F.classpower.chi:SetSize(F.classpower:GetWidth(), F.classpower:GetHeight())
		local curChi = UnitPowerMax("player", SPELL_POWER_CHI)

		for i = 1, curChi do
			F.classpower.chi[i]:SetSize((F.classpower.chi:GetWidth() - 16) / curChi, F.classpower.chi:GetHeight() - 6)
		end
	elseif SN.playerClass == "PRIEST" then
		F.classpower.shadoworbs:SetSize(F.classpower:GetWidth(), F.classpower:GetHeight())
		local curOrbs = PRIEST_BAR_NUM_ORBS

		for i = 1, curOrbs do
			F.classpower.shadoworbs[i]:SetSize((F.classpower.shadoworbs:GetWidth() - 16) / curOrbs, F.classpower.shadoworbs:GetHeight() - 6)
		end
	elseif SN.playerClass == "WARLOCK" then
		local spec = GetSpecialization()
		local number = F.classpower.shards.number

		F.classpower.shards:SetSize(F.classpower:GetWidth(), F.classpower:GetHeight())

		if spec == 1 or 3 then
			for i = 1, number do
				F.classpower.shards[i]:SetSize((F.classpower.shards:GetWidth() - 20) / number, F.classpower.shards:GetHeight() - 6)
			end
		elseif spec == 2 then
			F.classpower.shards[1]:SetSize((F.classpower.shards:GetWidth() - 11), F.classpower.shards:GetHeight() - 6)
		end
	end
end	

function CP:ClassReskin(tex)

	if SN.playerClass == "DEATHKNIGHT" then
		for i = 1, #F.classpower.runes do
			F.classpower.runes[i]:SetStatusBarTexture(LSM:Fetch("statusbar", tex))
		end
	elseif SN.playerClass == "PALADIN" then
		for i = 1, #F.classpower.holypower do
			F.classpower.holypower[i]:SetStatusBarTexture(LSM:Fetch("statusbar", tex))
		end
	elseif SN.playerClass == "DRUID" or SN.playerClass == "ROGUE" then
		if SN.playerClass == "DRUID" and GetSpecialization() == 1 then
			F.classpower.eclipse[1]:SetStatusBarTexture(LSM:Fetch("statusbar", tex))
			F.classpower.eclipse[2]:SetStatusBarTexture(LSM:Fetch("statusbar", tex))
		end
		for i = 1, #F.classpower.points do
			F.classpower.points[i]:SetStatusBarTexture(LSM:Fetch("statusbar", tex))
		end
	elseif SN.playerClass == "MONK" then
		for i = 1, #F.classpower.chi do
			F.classpower.chi[i]:SetStatusBarTexture(LSM:Fetch("statusbar", tex))
		end
	elseif SN.playerClass == "PRIEST" then
		for i = 1, #F.classpower.shadoworbs do
			F.classpower.shadoworbs[i]:SetStatusBarTexture(LSM:Fetch("statusbar", tex))
		end
	elseif SN.playerClass == "WARLOCK" then
		for i = 1, #F.classpower.shards do
			F.classpower.shards[i]:SetStatusBarTexture(LSM:Fetch("statusbar", tex))
		end
	end
end

function CP:PowerBubbles(maxbub)

	local frame = CreateFrame("Frame", nil, F.classpower)
	frame:SetSize(SN.data.profile.classResources.width, SN.data.profile.classResources.height)
	frame:SetPoint("CENTER", F.classpower, "CENTER", 0, 0)
	frame.maxbub = maxbub

	if (IsAddOnLoaded("ElvUI")) then
		SN:SNCreateBG(frame, "Frame")
	else
		frame:SetBackdrop({
			bgFile = M.media.blankTex, 
			edgeFile = M.media.blankTex, 
			tile = false, tileSize = 0, edgeSize = 2, 
			insets = { left = 2, right = 2, top = 2, bottom = 2 }
			})
		frame:SetBackdropColor(0.1, 0.1, 0.1, 1)
		frame:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
	end

	for i = 1, maxbub do
		frame[i] = CreateFrame("StatusBar", nil, frame)
		frame[i]:SetSize((SN.data.profile.classResources.width - (maxbub + 1) * 3) / maxbub, SN.data.profile.classResources.height - 6)
		if i == 1 then
			frame[i]:SetPoint("LEFT", frame, 3, 0)
		else
			frame[i]:SetPoint("LEFT", frame[(i - 1)], "RIGHT", 3, 0)
		end
		
		frame[i]:SetStatusBarTexture(LSM:Fetch("statusbar", SN.data.profile.classResources.texture))
		frame[i]:GetStatusBarTexture():SetHorizTile(false)
		frame[i]:SetMinMaxValues(0, 1)
		frame[i]:SetValue(1)
	end
	return frame
end