﻿------------------------------------------------------------------------------------------
-- Threat Frame---------------------------------------------------------------------------
------------------------------------------------------------------------------------------
local SN, L, F, T, M, _ = unpack(select(2,...))
local LSM = LibStub("LibSharedMedia-3.0")
local MODNAME = "threat"
local THREAT = SN:NewModule(MODNAME, "AceEvent-3.0")

local threat

local function getOptions()
	threat = {
		type = "group",
		name = L["Threat"],
		args = {
			enable = {
				type = "toggle",
				name = L["Enable Threat System"],
				order = 0,
				get = function(info)
					return SN.data.profile.threat[info[#info]]
				end,
				set = function(info, value)
					SN.data.profile.threat[info[#info]] = value
					if value then
						SN:EnableModule(MODNAME)
					else
						SN:DisableModule(MODNAME)
					end
				end,
			},
			h8 = {
				order = 3,
				type = "header",
				name = "Font",
				hidden = function() return not SN.data.profile.threat.enable end,
			},
			font = {
				type = "select",
				dialogControl = 'LSM30_Font',
				name = L["Font"],
				hidden = function() return not SN.data.profile.threat.enable end,
				order = 5,
				values = AceGUIWidgetLSMlists.font,
				get = function(info) return SN.data.profile.threat[info[#info]] end,--selectedClass end,
				set = function(info, value)
					SN.data.profile.threat[info[#info]] = value
					F.threat.value:SetFont(LSM:Fetch("font", value), SN.data.profile.threat.fontsize)
				end,
			},
			fontsize = {
				type = "range",
				name = L["Font Size of Threat Frame"],
				hidden = function() return not SN.data.profile.threat.enable end,
				order = 6,
				min = 10, max = 24, step = 1,
				get = function(info)
					return SN.data.profile.threat[info[#info]] 
				end,
				set = function(info, value)
					SN.data.profile.threat[info[#info]] = value
					F.threat.value:SetFont(LSM:Fetch("font", SN.data.profile.threat.font), value)
				end,
			},
		},
	}
	return threat
end

function THREAT:OnInitialize()
	SN:RegisterModuleOptions(MODNAME, getOptions, L["Threat"])

	F.threat = CreateFrame("Frame", threat, UIParent)
	F.threat:SetPoint(unpack(SN.data.profile.frames.threat))
	F.threat:SetSize(SN.data.profile.threat.width, SN.data.profile.threat.height)
	F.threat:SetClampedToScreen(true)

	F.threat:SetScript("OnEvent", function(self, event, ...)
		THREAT:ThreatEvents(self, event, ...)
	end)

	F.threat.value = F.threat:CreateFontString(nil, "ARTWORK")
	F.threat.value:SetFont(LSM:Fetch("font", SN.data.profile.threat.font), SN.data.profile.threat.fontsize, "OUTLINE")
	F.threat.value:SetTextColor(0.78, 0.25, 0.25)
	F.threat.value:SetText("")
	F.threat.value:SetAllPoints(F.threat)
	F.threat.value:SetJustifyH("CENTER")
	F.threat.value:SetJustifyV("MIDDLE")
	F.threat.name = "Threat Frame"
	F.threat.db = "threat"
	F.threat.storeMDtar = ""
	
end

function THREAT:OnEnable()
	F.threat:Show()
	F.threat:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
	F.threat:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	F.threat:RegisterEvent("PLAYER_REGEN_ENABLED")
	F.threat:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	F.threat:RegisterEvent("UNIT_SPELLCAST_SENT")
end

function THREAT:OnDisable()
	F.threat:UnregisterAllEvents()
	F.threat:Hide()
end

function THREAT:Lock()
	SN:LockFrame(F.threat)
	if SN.data.profile.threat.enable then
		F.threat:Show()
	end
end

function THREAT:Unlock()
	SN:UnlockFrame(F.threat)
end

function THREAT:Reset()
	local static = SN.DB.profile.frames
	local temp = SN.data.profile.frames
	F.threat:ClearAllPoints()

	F.threat:SetPoint(unpack(static.threat))
	wipe(temp.threat)
end

function THREAT:ApplySettings()
	F.threat:ClearAllPoints()
	F.threat:SetPoint(unpack(SN.data.profile.frames.threat))
	F.threat:SetSize(SN.data.profile.threat.width, SN.data.profile.threat.height)
	F.threat.value:SetFont(LSM:Fetch("font", SN.data.profile.threat.font), SN.data.profile.threat.fontsize, "OUTLINE")
end

function THREAT:ThreatEvents(self, event, ...)

	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		local _, type, _, sourceGUID, sourceName, _, _, destGUID, destName = ...
		local spellId, spellName = select(12, ...)

		if type == "SPELL_CAST_SUCCESS" then
			if sourceName == SN.playerName then
				--Checking to see if this success was a threat spell
				for i, threat in pairs(SN.data.profile.threat.spells) do
					if spellId == i then
						if threat.class == SN.playerClass then		
							if threat.msg == true then								
								SN:Announcer(self, nil, spellId, sourceName, destName, true)
							end
							self.value:SetText("")
						end
					end
				end
			end
		elseif type == "SPELL_CAST_FAILED" then
			--Checking to see if this success was a threat spell
			if (extraFlag1 == SPELL_FAILED_NOT_ON_MOUNTED) or (extraFlag1 == SPELL_FAILED_NOT_ON_SHAPESHIFT) then
				for i, spells in pairs(SN.data.profile.threat.spells) do
					if spellId == spells.id then
						if spells.msg == true then
							SN:Announcer(self, nil, spellId, sourceName, self.storeMDtar, false)
						end
					end
				end
			end
		end
	end

	if event == "UNIT_SPELLCAST_SENT" then
	--Store the target name for the misdirect/tricks of the trade since the failed event doesn't have a target arg
		local sourceName, spellId, _, destName = ...
		if sourceName == SN.playerName then
			for i, threat in pairs(SN.data.profile.threat.spells) do
				if spellId == i.id then
					if i.msg == true then
						self.storeMDtar = destName
					end
				end
			end
		end
	end
	if event == "UNIT_THREAT_LIST_UPDATE" or "UNIT_THREAT_SITUATION_UPDATE" then
		local enemy = ...
		local isTanking, status, _, _, threatvalue = UnitDetailedThreatSituation("player", enemy)
		
		if isTanking == 1 and status == 3 then return end
		
		if status == 1 then
			self.value:SetText(L["You pulled aggro."])
			for i, threat in pairs(SN.data.profile.threat.spells) do
				if i.class == SN.playerClass and i.order == 1 then
					local available = GetSpellCooldown(i)
					if available then
						self.value:SetText("\n"..L["You can use "]..i)
					end
				elseif i.class == SN.playerClass and i.order == 2 then
					local available = GetSpellCooldown(i)
					if available then
						self.value:SetText("\n"..L["You can use "]..i)
					end
				else
					SendChatMessage(SN.playerName..L[" pulled aggro on "]..enemy..L[" help me!"], "SAY")
				end
			end
		end
	end
	if event == "PLAYER_REGEN_ENABLED" then
		self.value:SetText("")
	end
end